`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer:   denis@strezhi.ru
// 
// Create Date:    23:54:33 11/22/2016 
// Design Name: 
// Module Name:    RTP_SENDER 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description:       , 
//   240     .   ,     
//     .
//       AXIA IP Audio (LiveWire)
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module RTP_SENDER(
    input clk,
    input signed [23:0] sample_left,
    input signed [23:0] sample_right,
	 input [7:0] fifo_used_count,
    output reg fifo_rd_en,
    output reg out_en = 0,
    output reg [7:0] tx_data
    );
	 
//reg [23:0]buf_sample; //  
reg [7:0]tx_data_buf;

reg [15:0]RTPsequence = 0; //   (p=p+1)
reg [31:0]RTPtimestamp = 0; //   (s=s+240)



localparam STATE_IDLE = 0;
localparam STATE_SEND_PACKET  = 1; // 
localparam STATE_TX_PLAYLOAD = 2; // 

// "IP source" - put an unused IP - if unsure, see comment below after the source code
parameter IPsource_1 = 10;
parameter IPsource_2 = 0;
parameter IPsource_3 = 9;
parameter IPsource_4 = 249;

// "IP destination" - put the IP of the PC you want to send to
parameter IPdestination_1 = 239;
parameter IPdestination_2 = 192;
parameter IPdestination_3 = 0; // AXIA chanel number part 1 !!!! 
parameter IPdestination_4 = 1; // AXIA chanel number part 2 !!!!


// "Physical Address source"
parameter MAC_source_1 = 8'h00;
parameter MAC_source_2 = 8'h40;
parameter MAC_source_3 = 8'hb5;
parameter MAC_source_4 = 8'h5e;
parameter MAC_source_5 = 8'h12; 
parameter MAC_source_6 = 8'h34; 

// "Physical Address destenation"
parameter MAC_destination_1 = 8'h01;
parameter MAC_destination_2 = 8'h00;
parameter MAC_destination_3 = 8'h5E;
parameter MAC_destination_4 = 8'h40;
parameter MAC_destination_5 = 8'h00; // AXIA chanel number part 1 !!!!
parameter MAC_destination_6 = 8'h01; // AXIA chanel number part 2 !!!!


// calculate the IP checksum, : http://fpga4fun.com/10BASE-T0.html : https://en.wikipedia.org/wiki/IPv4_header_checksum
parameter IPchecksum1 = 32'h00019C89 + (IPsource_1<<8)+IPsource_2+(IPsource_3<<8)+IPsource_4+
                                                                (IPdestination_1<<8)+IPdestination_2+(IPdestination_3<<8)+(IPdestination_4);
parameter IPchecksum2 =  ((IPchecksum1&32'h0000FFFF)+(IPchecksum1>>16));
parameter IPchecksum3 = ~((IPchecksum2&32'h0000FFFF)+(IPchecksum2>>16));


reg [3:0]state = STATE_IDLE;
reg [8:0]i=      0; //  
reg [2:0]part_sample= 0; //   
reg [7:0]n =      0; //    6   (  )
reg [10:0] rdaddress = 0;


always @(posedge clk)
begin
// Ethernet preamble
case(rdaddress)
  11'd0000: tx_data <= 8'h55;
  11'd0001: tx_data <= 8'h55;
  11'd0002: tx_data <= 8'h55;
  11'd0003: tx_data <= 8'h55;
  11'd0004: tx_data <= 8'h55;
  11'd0005: tx_data <= 8'h55;
  11'd0006: tx_data <= 8'h55;
  11'd0007: tx_data <= 8'hD5;
// Ethernet header
  11'd0008: tx_data <= MAC_destination_1;
  11'd0009: tx_data <= MAC_destination_2;
  11'd0010: tx_data <= MAC_destination_3;
  11'd0011: tx_data <= MAC_destination_4;
  11'd0012: tx_data <= MAC_destination_5;
  11'd0013: tx_data <= MAC_destination_6;
  11'd0014: tx_data <= MAC_source_1;
  11'd0015: tx_data <= MAC_source_2;
  11'd0016: tx_data <= MAC_source_3;
  11'd0017: tx_data <= MAC_source_4;
  11'd0018: tx_data <= MAC_source_5;
  11'd0019: tx_data <= MAC_source_6;
// IP header
  11'd0020: tx_data <= 8'h08;
  11'd0021: tx_data <= 8'h00;
  11'd0022: tx_data <= 8'h45;
  11'd0023: tx_data <= 8'hB8; //Diff Services ??
  11'd0024: tx_data <= 8'h05; // Total length 1 (0x05c8 = 1480 .  )
  11'd0025: tx_data <= 8'hc8; // Total length 2
  11'd0026: tx_data <= 8'hD0; //Identifier_1
  11'd0027: tx_data <= 8'hF8; //Identifier_2
  11'd0028: tx_data <= 8'h00; //fragmentation flag
  11'd0029: tx_data <= 8'h00; //fragmentation offset
  11'd0030: tx_data <= 8'h80; //TTL 128
  11'd0031: tx_data <= 8'h11; // protocol UDP 
  11'd0032: tx_data <= IPchecksum3[15:8]; // Header Checksum_1 
  11'd0033: tx_data <= IPchecksum3[ 7:0]; // Header Checksum_2   
  11'd0034: tx_data <= IPsource_1;
  11'd0035: tx_data <= IPsource_2;
  11'd0036: tx_data <= IPsource_3;
  11'd0037: tx_data <= IPsource_4;
  11'd0038: tx_data <= IPdestination_1;
  11'd0039: tx_data <= IPdestination_2;
  11'd0040: tx_data <= IPdestination_3;
  11'd0041: tx_data <= IPdestination_4;
// UDP header
  11'd0042: tx_data <= 8'h13; //Source Port_1: 5004 (0x138C)
  11'd0043: tx_data <= 8'h8C; //Source Port_2: 5004 (0x138C)
  11'd0044: tx_data <= 8'h13; //Destination Port_1: 5004 (0x138C)
  11'd0045: tx_data <= 8'h8C; //Destination Port_2: 5004 (0x138C)
  11'd0046: tx_data <= 8'h05; //Length_1 1460 (0x05B4)
  11'd0047: tx_data <= 8'hB4; //Length_2 1460 (0x05B4)
  11'd0048: tx_data <= 8'h00; //UDP Checksum_1:   AXIA    
  11'd0049: tx_data <= 8'h00; //UDP Checksum_2:   AXIA    
  11'd0050: tx_data <= 8'h80; //RTP header_1
  11'd0051: tx_data <= 8'h61; //RTP header_2
  11'd0052: tx_data <= RTPsequence[15:8]; //RTP Sequence_1    (p=p+1)
  11'd0053: tx_data <= RTPsequence[7:0]; //RTP Sequence_2    
  11'd0054: tx_data <= RTPtimestamp[31:24]; //RTP Time Stamp_1    (s=s+240)
  11'd0055: tx_data <= RTPtimestamp[23:16]; //RTP Time Stamp_2    
  11'd0056: tx_data <= RTPtimestamp[15:8]; //RTP Time Stamp_2    
  11'd0057: tx_data <= RTPtimestamp[7:0]; //RTP Time Stamp_2    
  11'd0058: tx_data <= 8'hEF; //Sync Src ID_1:  ???
  11'd0059: tx_data <= 8'hC0; //Sync Src ID_2:  ???
  11'd0060: tx_data <= 8'h00; //Sync Src ID_3:  AXIA chanel number part 1 !!!!
  11'd0061: tx_data <= 8'h01; //Sync Src ID_4:  AXIA chanel number part 2 !!!!
  default: tx_data <= tx_data_buf;
endcase




	case(state)
	STATE_IDLE: begin
		if( fifo_used_count > 239 )	//      (240)     . ggf
			state <= STATE_SEND_PACKET;
			rdaddress <=0; 
	//		buf_sample[23:0] <= sample_left[23:0]; // ?
		end
		
	STATE_SEND_PACKET:  begin

		
		out_en <=1;
		rdaddress <= rdaddress+1;
		if (rdaddress == 1502)begin
		out_en <=0;
		state <= STATE_IDLE;
		RTPsequence <= RTPsequence+1; //   (p=p+1)
		RTPtimestamp <= RTPtimestamp+240; //   (s=s+240)
		end
		
	//	if (rdaddress == 52) begin fifo_rd_en <=1; end
		if (rdaddress > 60 && rdaddress < 1501) begin //   60 
	//    if (part_sample == 0) begin fifo_rd_en <=1; end
	//	 else begin fifo_rd_en <=0; end
		 
		 if (part_sample == 5) begin
		 part_sample <= 0; 
		 end else begin 
		 part_sample <= part_sample +1;
		 end
		
		
		 if (part_sample == 4) fifo_rd_en <=1;
		 if (fifo_rd_en) fifo_rd_en <=0;



 case (part_sample)
      3'd0  : begin
                  tx_data_buf <= sample_left[23:16];
               end
      3'd1  : begin
                 tx_data_buf <= sample_left[15:8];
               end
      3'd2  : begin
                 tx_data_buf <= sample_left[7:0];
               end
      3'd3  : begin
                  tx_data_buf <= sample_right[23:16];
               end
      3'd4  : begin
                  tx_data_buf <= sample_right[15:8];
               end
      3'd5  : begin
                  tx_data_buf <= sample_right[07:0];
               end					
      default: begin
                  tx_data_buf <= 8'h00;
               end
endcase
		


		end 
		
	end
		

	endcase
end









endmodule
